local AddonName, NickAlert = ...

--[[ **************************************************************************
     * Add On Starts Here                                                     *
	 **************************************************************************
]]


local on = true
local off = false
local ReleaseNoteVer = "03/11/2014"

-- Local text colors
local yel = "|cFFFFFF00"
local wht = "|cFFFFFFFF"
local blu = "|cFF00FFFF"
local red = "|cFFFF0000"
local grn = "|cFF00FF00"
local ltblu = "|cFF00CCFF"
local ltred = "|cFFFF6060"
local res = "|r"


NickAlert.Defaults = {
	Toasts = on,
	TFriends = off, --All friends (on) or just marked (off)
	TGuild = off,  --All Guildies (on) or just marked (off)
	TBNet = off, --All BNet (on) or just marked (off)
	Guild = on, -- Enable on guild channel
	Party = on, -- Enable on party channel
	Say = on,  -- Enable for people near you
	BGs = on,
	Raid = on,
	Yell = on,
	Emotes = on,
	Chats = off,
	AllWhispers = off, --alert on ALL whispers
	AllFriends = off, --alert only friend whispers
	AllGuild = off, -- alert on all guild whispers
	AllBNet = off, --alert all bnet
	WhisperList = on,
	BNet = on,
	ChatDebug = off,
	CustomMP3 = off,
	CustomOGG = off,
	Throttle = on,
	ThrottleTime = 30,
	ThrottleAmount = 3,
	FadeDelay = 15, --fade message frame text after
	--
	-- Addon Output Settings
	DisplayMode = off, -- Display the mode of trigger
	MSBT = off,  -- Output achievement to Mik's Scrolling Battle Text
	Sound = on, -- Enable Sound Output
	MF = on, -- Message frame
	MFWarn = on, -- warn when Message Frame is unlocked
	MFAutoScale = on, -- Scale the frame to EffectiveScale() = 1 +/-
	MFScale = 100, -- Scale of message frame
	FontSize = 10, -- MF FontSize
	Log = on, -- Log frame logs all alerts while connected
	LogShow = off, -- Show log when you log in
	ScreenFlash = on, -- Flash Screen edges on an alert
	--
	-- Addon State
	Enable = on, -- enable/disable the addon 
	AutoAdd = on -- Auto add char nicks to list
	}


NickAlert_Settings = {}
local NAS -- alias for above

NickAlert_ChatFilter = {}
local CF -- alias for above

NickAlert_NickNames = {} -- Nicknames
local NN -- Alias for NickAlert_NickNames

NickAlert_WhisperList = {}
local WL -- Alias for NickAlert_WhisperList[Realm]

NickAlert_Toasts = {} -- Toast Table
local NAT 

NickAlert_PerChar = {}
local PerChar
NickAlert_PC = {}

NickAlert_Triggers = {}
local Triggers

NickAlert_AlertFilters = {}
local AF

-- Throttle Variables
local LastAlertTime
local ThrottleTable = {}
local Throttled = false

-- Message Frame Control
local MFShown = false
local MFShownTime = 0
local MFShowTimeWarn = 30

-- Toast stuff
local FriendOnline = ERR_FRIEND_ONLINE_SS:gsub( "|Hp[%p%a]*|h", "|Hp[%%p%%a]*|h") --"|Hplayer:%s|h[%s]|h has come online.";
local FriendOffline = ERR_FRIEND_OFFLINE_S:format("[%S]*")  --"%s has gone offline."

-- Bill's Utils
local addOptionMt = BillsUtils.addOptionMt
local StatColor = BillsUtils.StatColor
local SJprint = BillsUtils.SJprint
local OptSaveTF = BillsUtils.OptSaveTF
local OptSaveVal = BillsUtils.OptSaveVal
local EventToggle = BillsUtils.EventToggle
local CCC = BillsUtils.CCC
local Wait = BillsUtils.Wait

BillsUtils.Locals[#BillsUtils.Locals +1] = function ()
	addOptionMt = BillsUtils.addOptionMt
	StatColor = BillsUtils.StatColor
	SJprint = BillsUtils.SJprint
	OptSaveTF = BillsUtils.OptSaveTF
	OptSaveVal = BillsUtils.OptSaveVal
	EventToggle = BillsUtils.EventToggle
	CCC = BillsUtils.CCC
	Wait = BillsUtils.Wait
end

local ReleaseNotes = function()
	local Notes = { yel.."Nick Alert "..ReleaseNoteVer,
					"This version is a major re-write of this addon with new features and commands",
					"You can now have the edges of the screen flash when an alert is triggered.",
					"type "..blu.."/na screenflash on"..wht.." to enable the flash (default)",
					"type "..blu.."/na screenflash off"..wht.." to disable the flash",
					"You now have 2 separate lists of triggers to watch chats for, the old nicklist still searches case insensitive for a match at word starts",
					"The new Trigger list searches the messages case sensitive and using lua regex style matching as well for any match",
					"There are also new commands for whisper alerts, and BNet messages, and it should even alert you if a BNet friend who would BNet message alert you plain old WoW /tell's you",
					"You can even shut off the Message Frame alert if you don't like locking the frame",
					"Check out the control panel page for all the changes",
				}
				
	local count = #Notes
	for x = 1, count do
		NickAlert.ScrollingMessageFrame:AddMessage( "|cFFAAAAAA"..x.."/"..count.." |r"..Notes[x] ,1 ,1 ,1 )
	end
	NickAlert.LogFrame:Show()
	NickAlert.LogShown = true
	local Scroll, CScroll
	local done = false
	repeat
		NickAlert.ScrollingMessageFrame:ScrollUp()
		CScroll = NickAlert.ScrollingMessageFrame:GetCurrentScroll()
		if CScroll == Scroll then
			break
		end
		Scroll = CScroll
	until false
	NAS.ReleaseNoteVer = ReleaseNoteVer
end



NickAlert.Frame = CreateFrame("Frame", nil, UIParent)
NickAlert.Frame:RegisterEvent("ADDON_LOADED")
NickAlert.Frame:RegisterEvent("UI_SCALE_CHANGED")
NickAlert.Frame:RegisterEvent("PLAYER_REGEN_DISABLED")
NickAlert.Frame:RegisterEvent("PLAYER_REGEN_ENABLED")

NickAlert.EventManager = function()
	local Table = NickAlert_Settings
	local Frame = NickAlert.Frame
	local Enable = Table.Enable -- true if addon is enabled (false unregisters no matter what)
	
	local Event = { "CHAT_MSG_CHANNEL", "CHAT_MSG_EMOTE", "CHAT_MSG_TEXT_EMOTE", "CHAT_MSG_GUILD", "CHAT_MSG_OFFICER", "CHAT_MSG_PARTY", "CHAT_MSG_PARTY_LEADER", 
	"CHAT_MSG_RAID", "CHAT_MSG_RAID_LEADER", "CHAT_MSG_RAID_WARNING", "CHAT_MSG_SAY", "CHAT_MSG_YELL",
	"CHAT_MSG_WHISPER", "CHAT_MSG_BN_WHISPER", "CHAT_MSG_BN_CONVERSATION",
	"CHAT_MSG_SYSTEM", "BN_FRIEND_ACCOUNT_ONLINE", "BN_FRIEND_ACCOUNT_OFFLINE"}

	local Key = { "Chats", "Emotes", "Emotes", "Guild", "Guild", "Party", "Party", 
	"Raid", "Raid", "Raid", "Say", "Yell", "WhisperList", "BNet", "BNet",
	"Toasts", "Toasts", "Toasts"}

	for x = 1, #Event do
		EventToggle( Frame, Table, Key[x], Event[x], Enable )
	end
	
	local IChat = Table.BGs or Table.Party
	EventToggle( Frame, nil, IChat, "CHAT_MSG_INSTANCE_CHAT", Enable )
	EventToggle( Frame, nil, IChat, "CHAT_MSG_INSTANCE_CHAT_LEADER", Enable )
end

-- ****************************************************************************
-- * OnUpdate function manages the throttle table and other stuff

local e = 0
NickAlert.Frame:SetScript("OnUpdate", function(self, elapsed)
	e = e + elapsed
	if e < 1/5 then
		return
	end
	if MFShown and NAS.MFWarn then
		MFShownTime = MFShownTime + e
	end
	e = 0
		
	if #ThrottleTable > 0 then
		local CurrentTime = time()
		for x = #ThrottleTable, 1, -1 do
			if (CurrentTime - ThrottleTable[x]) > NAS.ThrottleTime then
				table.remove(ThrottleTable, x)
			end
		end
	end
	
	if Throttled then
		if #ThrottleTable < NAS.ThrottleAmount then
			Throttled = false
			if NAS.ChatDebug then
				SJprint(red, "Nick Alert: Throttle below threshold   Current:", #ThrottleTable, "   Max:", NAS.ThrottleAmount," over",NAS.ThrottleTime, "seconds", res)
			end
		end
	end
	
	if MFShown and NAS.MFWarn then
		if MFShownTime > MFShowTimeWarn then
			MFShownTime = 0
			NickAlert.MF:AddMessage("Move this frame to where you would like then type \"/namf lock\" to save the position. (Left button moves frame, Right button resizes frame)", 1, 1, 1 )
			PlaySound("READYCHECK", "Master")
		end
	end
end)

local Flash = function()
	if not NickAlert.FlashFrame then
		local flasher = CreateFrame("Frame", "NAlertFlashFrame")
		flasher:SetToplevel(true)
		flasher:SetFrameStrata("FULLSCREEN_DIALOG")
		flasher:SetAllPoints(UIParent)
		flasher:EnableMouse(false)
		flasher:Hide()
		flasher.texture = flasher:CreateTexture(nil, "BACKGROUND")
		flasher.texture:SetTexture("Interface\\addons\\nickalert\\test2.tga")
		--flasher.texture:SetTexture("Interface\\FullScreenTextures\\OutOfControl")
		flasher.texture:SetAllPoints(UIParent)
		flasher.texture:SetBlendMode("ADD")
		flasher:SetScript("OnShow", function(self)
			self.elapsed = 0
			self:SetAlpha(0)
		end)
		flasher:SetScript("OnUpdate", function(self, elapsed)
			elapsed = self.elapsed + elapsed
			if elapsed < 2.6 then
				local alpha = elapsed % 1.3
				if alpha < 0.15 then
					self:SetAlpha(alpha / 0.15)
				elseif alpha < 0.9 then
					self:SetAlpha(1 - (alpha - 0.15) / 0.6)
				else
					self:SetAlpha(0)
				end
			else
				self:Hide()
			end
			self.elapsed = elapsed
		end)
		NickAlert.FlashFrame = flasher
	end

	NickAlert.FlashFrame:Show()
end

-- ****************************************************************************
-- * Event Handler
local eventHandler = {}
local LastMsgLine = ""


NickAlert.Frame:SetScript("OnEvent",function(self, event, ...)
	if NickAlert_Settings and NickAlert_Settings.ChatDebug then
		local args = {...}
		print("\nNickAlert Chat Debug:")
		print("event = "..event)
		for x = 1, #args do
			print(tostring(x).." = ", (args[x] == "" and "\"\"" or args[x]))
		end
		print("\n")
	end
	if event ~= "ADDON_LOADED" and not(NAS.Enable) then return end
	if string.find( event, "^CHAT_MSG" ) then
		local msgLine = ( select( 11, ...) )
		if msgLine then 
			if msgLine == LastMsgLine then
				return 
			end
			LastMsgLine = msgLine
		end
	end
	eventHandler[event](self, event, ...)
end)

-- "ADDON_LOADED"
eventHandler["ADDON_LOADED"] = function(self, event, EventAddonName)
	if EventAddonName == AddonName then
		SJprint(grn, "Nick Alert v", GetAddOnMetadata( AddonName, "Version"), "loaded",  res)
		addOptionMt( NickAlert_Settings , NickAlert.Defaults )
		NAS = NickAlert_Settings
	
		NickAlert.EventManager()
	
		local realm = GetRealmName()
		local player = UnitName("player")
		
		if not( NickAlert_NickNames[realm] ) then
			NickAlert_NickNames[realm] = {}
		end
		NN = NickAlert_NickNames[realm]


		if not NickAlert_WhisperList[realm] then
			NickAlert_WhisperList[realm] = {}
		end
		
		WL = NickAlert_WhisperList[realm]
		CF = NickAlert_ChatFilter
		
		PerChar = NickAlert_PerChar
		if not(PerChar.Toasts) then
			PerChar.Toasts = {}
		end

		-- Var upgrade section
		if NickAlert_PC[realm] and NickAlert_PC[realm][player] then
			for index, value in pairs( NickAlert_PC[realm][player]) do
				if not(PerChar[index]) then
					PerChar[index] = value
				end
			end
			NickAlert_PC[realm][player] = nil
			local count = 0 
			for _, _ in pairs(NickAlert_PC[realm]) do
				count = count + 1
			end
			if count == 0 then 
				NickAlert_PC[realm] = nil
			end
			count = 0 
			for _, _ in pairs( NickAlert_PC ) do
				count = count + 1
			end
			if count == 0 then 
				NickAlert_PC = nil
			end
		end
		-- end of ver upgrade
			
		if not(PerChar.MsgFrameSet) and NAS.MF then
			PerChar.MsgFrameSet = true
			NickAlert.MsgFrameUnlock()
			NickAlert.MF:AddMessage("Move this frame to where you would like then type \"/namf lock\" to save the position", 1, 1, 1 )
		else
			NickAlert.MsgFrameLock()
		end
	
		if not(PerChar.NickAdded) and NAS.AutoAdd then
			PerChar.NickAdded = true
			NickAlert.List( "NickList", "add", player)
		end
		
		NAT = NickAlert_Toasts
		Triggers = NickAlert_Triggers
		
		AF = NickAlert_AlertFilters
		
		-- put the tables into addon table
		NickAlert.Settings = NAS
		NickAlert.NickNames = NN
		NickAlert.ChatFilter = CF
		NickAlert.WhisperList = WL
		NickAlert.ToastsList = NAT
		NickAlert.Triggers = Triggers
		NickAlert.AlertFilters = AF
	
		NickAlert.MF:SetFont( STANDARD_TEXT_FONT, NAS.FontSize, "OUTLINE")
		NickAlert.MF:SetScale(NAS.MFScale / 100 )
		
		-- release notes
		if NAS.ReleaseNoteVer ~= ReleaseNoteVer then
			Wait(5, ReleaseNotes)
		end
	end
	return self, event, EventAddonName
end

-- "UI_SCALE_CHANGED"	
eventHandler["UI_SCALE_CHANGED"] = function()
	if NAS.MFAutoScale then
		local UIScale = UIParent:GetScale()
		NAS.MFScale =  math.floor((1/UIScale)*100)
		SJprint(yel, "NickAlert:", wht, "message frame scale auto adjusted", res)
		NickAlert.MF:SetScale(NAS.MFScale / 100 )
		NickAlert.LogFrame:SetScale(NAS.MFScale / 100 )
	end
	return
end
	
-- "PLAYER_REGEN_DISABLED"
eventHandler["PLAYER_REGEN_DISABLED"] = function()
	if NickAlert.LogShown then
		NickAlert.LogFrame:Hide()
	end
	return
end

-- "PLAYER_REGEN_ENABLED"
eventHandler["PLAYER_REGEN_ENABLED"] = function()
	if NickAlert.LogShown then
		NickAlert.LogFrame:Show()
	end
	return
end

-- "CHAT_MSG_WHISPER"
eventHandler["CHAT_MSG_WHISPER"] = function(self, event, msg, name, _, _, _, _, _, _, _, _, msgLine, guid)
	local TempName, TempRealm = string.split("-", name)
	local announce = false
	if NAS.AllWhispers then
		announce = true
	elseif NAS.AllFriends then
		local friend = GetFriendInfo( TempName )
		if friend and TempRealm == GetRealmName() then
			announce = true
		end
	elseif NAS.WhisperList and #WL > 0 then
		for x = 1, #WL do
			if string.find(string.lower(name), "^"..string.lower(WL[x])) then
				announce = true
				break
			end
		end
	end
	if announce then
		NickAlert.Alert(event, msg, name, "TELL")
		return
	end
	
	if NAS.BNet or NAS.AllBNet then
		local numBNetTotal, numBNetOnline = BNGetNumFriends()
		if numBNetOnline > 0 then
			local presenceID, presenceName, battleTag, isBattleTagPresence, toonName, toonID, client, isOnline, lastOnline, isAFK, isDND, messageText, noteText, isRIDFriend, broadcastTime, canSoR 
			for x = 1, numBNetTotal do
				presenceID, presenceName, battleTag, isBattleTagPresence, toonName, toonID, client, isOnline, lastOnline, isAFK, isDND, messageText, noteText, isRIDFriend, broadcastTime, canSoR = BNGetFriendInfo(x)
				local _, _, _, realmName = BNGetToonInfo(presenceID)
				if TempName == toonName and TempRealm == realmName then
					if NAS.AllBNet or string.find(noteText, "NAlert") then
						NickAlert.Alert(event, msg, name, "TELL-BN")
					end
					return
				end
			end
		end
	end
	
	if NAS.AllGuild and IsInGuild() then
		local count = GetNumGuildMembers()
		for x = 1, count do
			local Guildie = GetGuildRosterInfo(x)
			if Guildie == name then
				NickAlert.Alert(event, msg, name, "TELL-/G")
				return
			end
		end
	end
end

-- "CHAT_MSG_BN_WHISPER" "CHAT_MSG_BN_CONVERSATION"
eventHandler["CHAT_MSG_BN_WHISPER"] = function(self, event, msg, name, _, _, _, _, _, _, _, _, msgLine, _, presenceID, _)
	if not(NAS.BNet) then return end
	local _, _, _, _, _, _, _, _, _, _, _, _, noteText, _, _ = BNGetFriendInfoByID(presenceID)
	if NAS.AllBNet or (noteText and string.find( noteText, "NAlert")) then
		NickAlert.Alert(event, msg, name, "BNet")
	end
end
eventHandler["CHAT_MSG_BN_CONVERSATION"] = eventHandler["CHAT_MSG_BN_WHISPER"]

-- "CHAT_MSG_INSTANCE_CHAT" "CHAT_MSG_INSTANCE_CHAT_LEADER"
eventHandler["CHAT_MSG_INSTANCE_CHAT"] = function(self, event, msg, name, _, _, _, _, _, _, channel, _, msgLine)
	local IsLeader = string.find(event, "LEADER")
	if IsInRaid() then
		if not(NAS.Raid) then return end
		event = "CHAT_MSG_RAID"
	else
		if not(NAS.Party) then return end
		event = "CHAT_MSG_PARTY"
	end
	if IsLeader then
		event = event.."_LEADER"
	end
	
	if #NN > 0 then
		local tempMsg = string.lower(msg)
		local tempNN
		for x = 1, #NN do
			tempNN = string.lower( NN[x] )
			if string.find( tempMsg, "^"..tempNN ) or string.find( tempMsg, "[%A]"..tempNN ) then
				NickAlert.Alert(event, msg, name, "/I")
				return
			end
		end
	end
	if #Triggers > 0 then
		for x  = 1, #Triggers do
			if string.find(msg, Triggers[x]) then
				NickAlert.Alert(event, msg, name, "/I")
				return
			end
		end
	end
end
eventHandler["CHAT_MSG_INSTANCE_CHAT_LEADER"] = eventHandler["CHAT_MSG_INSTANCE_CHAT"]

-- "CHAT_MSG_CHANNEL" "CHAT_MSG_EMOTE" "CHAT_MSG_TEXT_EMOTE" "CHAT_MSG_GUILD" "CHAT_MSG_OFFICER"
-- "CHAT_MSG_PARTY" "CHAT_MSG_PARTY_LEADER" "CHAT_MSG_RAID" "CHAT_MSG_RAID_LEADER" "CHAT_MSG_RAID_WARNING"
-- "CHAT_MSG_SAY" "CHAT_MSG_YELL"
eventHandler["CHAT_MSG_CHANNEL"] = function(self, event, msg, name, _, _, _, _, _, _, channel, _, msgLine)
	local events = { ["CHAT_MSG_CHANNEL"] = 1, ["CHAT_MSG_EMOTE"] = 2, ["CHAT_MSG_TEXT_EMOTE"] = 3, ["CHAT_MSG_GUILD"] = 4, ["CHAT_MSG_OFFICER"] = 5,
		             ["CHAT_MSG_PARTY"] = 6, ["CHAT_MSG_PARTY_LEADER"] = 7, ["CHAT_MSG_RAID"] = 8, ["CHAT_MSG_RAID_LEADER"] = 9, ["CHAT_MSG_RAID_WARNING"] = 10,
		             ["CHAT_MSG_SAY"] = 11, ["CHAT_MSG_YELL"] = 12,
					}
	local vars = { "Chats", "Emotes", "Emotes", "Guild", "Guild", 
	               "Party", "Party", "Raid", "Raid", "Raid",
				   "Say", "Yell",
				 }
				 
	local mode = { channel, "/EM", "/EM", "/G", "/O",
				   "/P", "/P", "/R", "/R", "/RW",
				   "/S", "/Y",
				 }
	
	local varname = vars[events[event]]
	local varmode = mode[events[event]]
	
	if not(NAS[varname]) then return end
		
	if event == "CHAT_MSG_CHANNEL" then  -- arg9 channel name without number (this is _sometimes_ in lowercase)
		local found = false
		if #CF > 0 then
			channel = string.lower(channel)
			for x = 1, #CF do
				if string.find( channel, string.lower(CF[x]) )then
					found = true
					break
				end
			end
		end
		if not(found) then return end
	end
	
	if #NN > 0 then
		local tempMsg = string.lower(msg)
		local tempNN
		for x = 1, #NN do
			tempNN = string.lower( NN[x] )
			if string.find( tempMsg, "^"..tempNN ) or string.find( tempMsg, "[%A]"..tempNN ) then
				NickAlert.Alert(event, msg, name, varmode)
				return
			end
		end
	end
	
	if #Triggers > 0 then
		for x  = 1, #Triggers do
			if string.find(msg, Triggers[x]) then
				NickAlert.Alert(event, msg, name, varmode)
				return
			end
		end
	end
end
eventHandler["CHAT_MSG_EMOTE"] = eventHandler["CHAT_MSG_CHANNEL"]
eventHandler["CHAT_MSG_TEXT_EMOTE"] = eventHandler["CHAT_MSG_CHANNEL"]
eventHandler["CHAT_MSG_GUILD"] = eventHandler["CHAT_MSG_CHANNEL"]
eventHandler["CHAT_MSG_OFFICER"] = eventHandler["CHAT_MSG_CHANNEL"]
eventHandler["CHAT_MSG_PARTY"] = eventHandler["CHAT_MSG_CHANNEL"]
eventHandler["CHAT_MSG_PARTY_LEADER"] = eventHandler["CHAT_MSG_CHANNEL"]
eventHandler["CHAT_MSG_RAID"] = eventHandler["CHAT_MSG_CHANNEL"]
eventHandler["CHAT_MSG_RAID_LEADER"] = eventHandler["CHAT_MSG_CHANNEL"]
eventHandler["CHAT_MSG_RAID_WARNING"] = eventHandler["CHAT_MSG_CHANNEL"]
eventHandler["CHAT_MSG_SAY"] = eventHandler["CHAT_MSG_CHANNEL"]
eventHandler["CHAT_MSG_YELL"] = eventHandler["CHAT_MSG_CHANNEL"]

-- "CHAT_MSG_SYSTEM"
eventHandler["CHAT_MSG_SYSTEM"] = function(self, event, msg, name, _, _, _, _, _, _, _, _, msgLine)
	if not(NAS.Toasts) then return end
	local _
	if string.find(msg, FriendOnline) then -- mode = Online
		_, _, name = string.find(msg, "|Hplayer:([%a%-]*)|h")
	elseif string.find(msg, FriendOffline) then
		_, _, name = string.find( msg, ERR_FRIEND_OFFLINE_S:format("([%a%-]*)"))
	else 
		return
	end
	
	if GetFriendInfo( name ) then -- Alert Type TFriends
		if NAS.TFriends or PerChar.Toasts[name] then
			NickAlert.Alert(event, msg, name, "Toast")
		end
	else -- Alert Type TGuild
		local MyGuild = (GetGuildInfo("player"))
		if not(NAT[MyGuild]) then NAT[MyGuild] = {} end
		if not(string.find("-", name)) then
			name = name.."-"..GetRealmName()
		end
		if NAS.TGuild or NAT[MyGuild][name] then
			NickAlert.Alert(event, msg, name, "Toast")
		end
	end
end

-- "BN_FRIEND_ACCOUNT_ONLINE"
eventHandler["BN_FRIEND_ACCOUNT_ONLINE"] = function(self, event, presenceID)
	if not(NAS.Toasts) then return end
	local _, Name, BattleTag, _, toonName, toonID, client, _, _, _, _, _, noteText = BNGetFriendInfoByID( presenceID )
	if NAS.TBNet or (noteText and string.find(noteText, "NAToast")) then
		local msg
		if client == "WoW" then
			local _, _, _, realmName, _, faction, _, class, _, zoneName, level, _, _, _, _, _ = BNGetToonInfo( presenceID)
			class = string.upper(class)
			if string.find(class, " ") then
				local c1, c2 = string.split(class, " ")
				class = c1..c2
			end
			local color = GetQuestDifficultyColor(tonumber(level))
			color.r = math.floor(color.r * 255)
			color.g = math.floor(color.g * 255)
			color.b = math.floor(color.b * 255)
				
			msg = string.format("%s%s %slvl. |cFF%02X%02X%02X%02d %s|Hplayer:%s|h%s|h %s%s |cFFFFFFFF%s", ltred, Name, wht, color.r, color.g, color.b, level, CCC(class), toonName, toonName, (faction == FACTION_ALLIANCE and ltblu or ltred), faction, BN_TOAST_ONLINE)
		else
			if Name == toonName then
				msg = Name.." "..BN_TOAST_ONLINE
			else
				msg = Name.." "..toonName.." "..BN_TOAST_ONLINE
			end
		end
		NickAlert.Alert(event, msg, Name, "Toast")
	end
end

-- "BN_FRIEND_ACCOUNT_OFFLINE"
eventHandler["BN_FRIEND_ACCOUNT_OFFLINE"] = function(self, event, presenceID)
	if not(NAS.Toasts) then return end
	local _, Name, BattleTag, _, toonName, toonID, client, _, _, _, _, _, noteText = BNGetFriendInfoByID( presenceID )
	if NAS.TBNet or (noteText and string.find(noteText, "NAToast")) then
		local msg = Name.." "..BN_TOAST_OFFLINE
		NickAlert.Alert(event, msg, Name, "Toast")
	end
end

NickAlert.Alert = function(event, msg, name, mode)
	if name and UnitIsUnit("player", name) and not(NAS.ChatDebug) then return end
	
	if #AF > 0 and not(NAS.ChatDebug) then
		for x = 1, #AF do
			if string.find( msg, AF[x] ) then
				return
			end
		end
	end
	
	if string.find(event, "CHAT_MSG_") then
		event = string.sub(event, 10)
	end
	local color
	
	if ChatTypeInfo[event] then
		color = ChatTypeInfo[event]
	elseif event == "BN_FRIEND_ACCOUNT_ONLINE" then
		color = {r = 1, g = 0, b = 0}
	elseif event == "BN_FRIEND_ACCOUNT_OFFLINE" then
		color = {r = 1, g = 0, b = 0}
	else
		color = {r = 1, g = 1, b = 1}
	end
	
	if Throttled then
		if NAS.ChatDebug then
			SJprint(red, "Nick Alert: Alert Throttled ", #ThrottleTable, "/", NAS.ThrottleAmount, res)
		end
		return
	end
	
	if NAS.Throttle then
		ThrottleTable[#ThrottleTable + 1] = time()
		if #ThrottleTable > NAS.ThrottleAmount then
			Throttled = true
			if NAS.ChatDebug then
				SJprint(red, "Nick Alert: Alert Throttled ", #ThrottleTable, "/", NAS.ThrottleAmount, res)
			end
			return
		end
	end
	
	local outtext
	if mode == "Toast" then
		outtext = msg
	else
		outtext = string.join( " ", name, ":", msg)
	end
	
	if NAS.DisplayMode then
		outtext = string.join(" ", mode, outtext)
	end
	
	if NAS.ChatDebug then
		SJprint( yel, "Nick Alert: Chat Debug ", wht, outtext, res )
	end
	
	if NAS.ScreenFlash then
		Flash()
	end
	
	if NAS.MSBT and IsAddOnLoaded("MikScrollingBattleText") then
		if not(MikSBT.IsModDisabled()) then
			MikSBT.DisplayMessage( outtext , MikSBT.DISPLAYTYPE_NOTIFICATION, true, R*255 , G*255, B*255)
		end
	end

	if NAS.Sound then
		if NAS.CustomMP3 then
			PlaySoundFile("Interface\\AddOns\\NickAlert_CustomSounds\\alert.mp3", "Master")
		elseif NAS.CustomOGG then
			PlaySoundFile("Interface\\AddOns\\NickAlert_CustomSounds\\alert.ogg", "Master")
		else
			PlaySound("READYCHECK", "Master")
		end
	end
	
	if NAS.MF then
		NickAlert.MF:AddMessage(outtext, R, G, B )
	end
	
	if NAS.Log then
		NickAlert.ScrollingMessageFrame:AddMessage( date("%H:%M:%S  ")..outtext, R, G, B)
	end

end


local Backdrop={
	bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
	edgeFile="Interface\\DialogFrame\\UI-DialogBox-Border",
	tile = true,
	tileSize = 16,
	edgeSize = 16,
	insets = {
		left = 0,
		right = 0,
		top = 0,
		bottom = 0
	}
}

local OnMouseDown = function(self, button)
	if button == "LeftButton" then
		self:StartMoving()
		self.isMoving = true
		self.hasMoved = false
	elseif button == "RightButton" then
		self:StartSizing()
		self.isMoving = true
		self.hasMoved = false
	end
end

local OnMouseUp = function(self)
    if ( self.isMoving ) then
        self:StopMovingOrSizing();
        self.isMoving = false;
        self.hasMoved = true;
    end
end

NickAlert.MF = CreateFrame("MessageFrame", "NAMsgFrame", UIParent)
local MF = NickAlert.MF

MF:SetFrameStrata("DIALOG")
MF:SetToplevel(true)
MF:SetSize(UIParent:GetWidth() / 4, 50)
MF:SetTimeVisible(15)
MF:SetFadeDuration(5)
MF:SetFading(true)
MF:SetClampedToScreen(true)
MF:SetInsertMode("TOP")
MF:SetIndentedWordWrap(true)
MF:SetFont( STANDARD_TEXT_FONT, 10)
MF:SetJustifyH("LEFT")
MF:SetPoint("CENTER", UIParent, "CENTER" )
MF:Show()
MF:SetMovable(true)
MF:SetResizable(true)
MF:SetMinResize( 30, 30 )
MF:RegisterForDrag("LeftButton","RightButton")
MF:SetScript("OnMouseDown",OnMouseDown)
MF:SetScript("OnMouseUp",OnMouseUp)
MF.Title = MF:CreateFontString( nil , "OVERLAY", "GameFontHighlight")
MF.Title:SetPoint( "TOP", MF, "TOP", 0, 8 )
MF.Title:SetJustifyH( "CENTER" )
MF.Title:SetJustifyV( "MIDDLE" )
MF.Title:SetText("Nick Alert")
MF:SetScale( 1 )

NickAlert.MsgFrameLock = function()
	MF:EnableMouse(false)
	MF:SetBackdrop( nil )
	MF.Title:Hide()
	MFShown = false
end

NickAlert.MsgFrameUnlock = function()
	MF:EnableMouse(true)
	MF:SetBackdrop( Backdrop )
	MF.Title:Show()
	NickAlert.MF:AddMessage( yel.."Nick Alert "..wht.." Left Button moves the frame, Right button resizes the frame. Type \"/namf lock\" when you are done"..res )
	MFShown = true
	MFShownTime = 0
end

